@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
- * Anders Kvist <akv@lnxbx.dk>
+ * * Copyright (C) 2006-2011 Anders Brander <anders@brander.dk>, 
+ * * Anders Kvist <akv@lnxbx.dk> and Klaus Post <klauspost@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,11 +19,14 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#ifdef WIN32
+#include <pthread.h> /* MinGW WIN32 gmtime_r() */
+#endif
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <string.h>
 #include "config.h"
-#include "rawstudio.h"
+#include "application.h"
 #include "filename.h"
 #include "conf_interface.h"
 #include "gettext.h"
@@ -43,6 +46,7 @@ filename_parse(const gchar *in, const gchar *filename, const gint snapshot)
 	 * %s = setting (A,B or C)
 	 * %d = date (will have to wait until read from exif)
 	 * %t = time (will have to wait until read from exif)
+	 * %p = path of raw file
 	 */
 
 	gchar temp[1024];
@@ -54,12 +58,14 @@ filename_parse(const gchar *in, const gchar *filename, const gint snapshot)
 	gboolean file_exists = FALSE;
 	gint i = 1;
 	gchar *basename;
+	gchar *path;
 	RSMetadata *metadata = rs_metadata_new_from_file(filename);
 
 	if (filename == NULL) return NULL;
 	if (in == NULL) return NULL;
 
 	basename = g_path_get_basename(filename);
+	path = g_path_get_dirname(filename);
 	output = g_strrstr(basename, ".");
 
 	/* Prepare time/date */
@@ -164,6 +170,42 @@ filename_parse(const gchar *in, const gchar *filename, const gint snapshot)
 							g_free(result);
 							break;
 						}
+						case 'D':
+						{
+							gchar *result = g_new0(gchar, 500);
+							switch (in[n+2])
+							{
+								case 'y':
+									strftime(result, 500, "%y", tm);
+									break;
+								case 'Y':
+									strftime(result, 500, "%Y", tm);
+									break;
+								case 'm':
+									strftime(result, 500, "%m", tm);
+									break;
+								case 'M':
+									strftime(result, 500, "%B", tm);
+									break;
+								case 'd':
+									strftime(result, 500, "%d", tm);
+									break;
+								case 'D':
+									strftime(result, 500, "%A", tm);
+									break;
+								default:
+									*result = 0;
+								  
+							}
+							strcpy(&temp[m], result);
+
+							n += 3;
+							m += strlen(result);
+
+							g_free(result);
+							break;
+						}
+
 						case 't':
 						{
 							gchar *result = g_new0(gchar, 9);
@@ -176,6 +218,13 @@ filename_parse(const gchar *in, const gchar *filename, const gint snapshot)
 							g_free(result);
 							break;
 						}
+						case 'p':
+						{
+							strcpy(&temp[m], path);
+							m += strlen(path);
+							n += 2;
+							break;
+						}
 						default:
 							temp[m++] = in[n];
 							temp[m++] = in[n+1];
@@ -210,6 +259,7 @@ filename_parse(const gchar *in, const gchar *filename, const gint snapshot)
 	
 	g_free(basename);
 	g_free(tm);
+	g_free(path);
 	g_object_unref(metadata);
 	
 	return output;
@@ -238,32 +288,40 @@ filename_entry_changed_writeconf(GtkEntry *entry, gpointer user_data)
 	return;
 }
 
-static void add_f(GtkMenuItem *menuitem, GtkEntry *entry) { gtk_entry_append_text(entry, "%f"); };
-static void add_c(GtkMenuItem *menuitem, GtkEntry *entry) { gtk_entry_append_text(entry, "%2c"); };
-static void add_s(GtkMenuItem *menuitem, GtkEntry *entry) { gtk_entry_append_text(entry, "%s"); };
+#define TEXT_FUNCTION(A,B) static void A(GtkMenuItem *menuitem, GtkBin *combo) {\
+	GtkWidget *entry = gtk_bin_get_child(combo);\
+	gtk_entry_append_text(GTK_ENTRY(entry), B); }
 
-static void
-add_d(GtkMenuItem *menuitem, GtkBin *combo)
-{
-	GtkWidget *entry = gtk_bin_get_child(combo);
-	gtk_entry_append_text(GTK_ENTRY(entry), "%d");
-};
+TEXT_FUNCTION(add_f,"%f");
+TEXT_FUNCTION(add_c,"%2c");
+TEXT_FUNCTION(add_s,"%s");
+TEXT_FUNCTION(add_d,"%d");
+TEXT_FUNCTION(add_t,"%t");
+TEXT_FUNCTION(add_p,"%p");
+TEXT_FUNCTION(add_DY,"%DY");
+TEXT_FUNCTION(add_Dy,"%Dy");
+TEXT_FUNCTION(add_Dm,"%Dm");
+TEXT_FUNCTION(add_DM,"%DM");
+TEXT_FUNCTION(add_DD,"%DD");
+TEXT_FUNCTION(add_Dd,"%Dd");
 
-static void
-add_t(GtkMenuItem *menuitem, GtkBin *combo)
-{
-	GtkWidget *entry = gtk_bin_get_child(combo);
-	gtk_entry_append_text(GTK_ENTRY(entry), "%t");
-};
+#undef TEXT_FUNCTION
 
 static void
 filename_add_clicked(GtkButton *button, gpointer user_data)
 {
 	gui_menu_popup(GTK_WIDGET(button), user_data,
 		_("%f - Original filename"), add_f,
+		_("%p - Path of original file"), add_p,
 		_("%2c - Incremental counter"), add_c,
 		_("%s - Setting id (A, B or C)"), add_s,
 		_("%d - Date from EXIF (YYYY-MM-DD)"), add_d,
+		_("%DY - Year from EXIF (YYYY)"), add_DY,
+		_("%Dy - Year from EXIF (YY)"), add_Dy,
+		_("%Dm - Month from EXIF (MM)"), add_Dm,
+		_("%DM - Month from EXIF (Text)"), add_DM,
+		_("%Dd - Date from EXIF (DD)"), add_Dd,
+		_("%DD - Day of week from EXIF"), add_DD,
 		_("%t - Time from EXIF (HH:MM:SS)"), add_t,
 		-1
 	);
@@ -272,13 +330,28 @@ filename_add_clicked(GtkButton *button, gpointer user_data)
 GtkWidget *
 rs_filename_chooser_button_new(gchar **filename, const gchar *conf_key)
 {
+	gint i;
+	const static gchar *filenames[] = {
+		DEFAULT_CONF_EXPORT_FILENAME,
+		"%f",
+		"%f_%c",
+		"%f_output_%4c",
+		NULL };
 	GtkWidget *addbutton;
 	GtkWidget *hbox = gtk_hbox_new(FALSE, 2);
+	GtkWidget *combo;
 	GtkWidget *entry;
 
+	combo = gtk_combo_box_entry_new_text();
+	entry = gtk_bin_get_child(GTK_BIN(combo));
 	addbutton = gtk_button_new_with_label("+");
 
-	entry = gtk_entry_new();
+	g_object_set_data(G_OBJECT(hbox), "entry", entry);
+
+	for(i=0; filenames[i] != NULL; i++)
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), filenames[i]);	
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
+
 	if (filename)
 	{
 		gtk_entry_set_text(GTK_ENTRY(entry), *filename);
@@ -288,10 +361,20 @@ rs_filename_chooser_button_new(gchar **filename, const gchar *conf_key)
 	if (conf_key)
 		g_signal_connect (G_OBJECT(entry), "changed",
 			G_CALLBACK(filename_entry_changed_writeconf), (gpointer) conf_key);
-	g_signal_connect (G_OBJECT(addbutton), "clicked",
-		G_CALLBACK(filename_add_clicked), (gpointer) entry);
 
-	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+	/* If we have a conf_key but no supplied filename, we try to load saved
+	 * filename from conf system */
+	if (conf_key && !filename)
+	{
+		gchar *saved_filename;
+		saved_filename = rs_conf_get_string(conf_key);
+		if (saved_filename)
+			gtk_entry_set_text(GTK_ENTRY(entry), saved_filename);
+	}
+	g_signal_connect (G_OBJECT(addbutton), "clicked",
+		G_CALLBACK(filename_add_clicked), combo);
+
+	gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), addbutton, FALSE, FALSE, 0);
 
 	return(hbox);
