@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
- * Anders Kvist <akv@lnxbx.dk>
+ * * Copyright (C) 2006-2011 Anders Brander <anders@brander.dk>, 
+ * * Anders Kvist <akv@lnxbx.dk> and Klaus Post <klauspost@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +17,18 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#ifndef WIN32
+#include <gconf/gconf-client.h>
+#endif
+
 typedef struct _RS_CONFBOX RS_CONFBOX;
 
+#define gui_label_set_text_printf(label, format, ...) do { \
+	gchar *__new_text = g_strdup_printf(format, __VA_ARGS__); \
+	gtk_label_set_text(label, __new_text); \
+	g_free(__new_text); \
+} while (0)
+
 extern gpointer gui_confbox_get_active(RS_CONFBOX *confbox);
 extern void gui_confbox_add_entry(RS_CONFBOX *confbox, const gchar *conf_id, const gchar *text, gpointer *user_data);
 extern void gui_confbox_load_conf(RS_CONFBOX *confbox, gchar *default_value);
@@ -34,15 +44,12 @@ extern void gui_tooltip_window(GtkWidget *widget, gchar *tip_tip, gchar *tip_pri
 extern void gui_batch_directory_entry_changed(GtkEntry *entry, gpointer user_data);
 extern void gui_batch_filename_entry_changed(GtkComboBox *combobox, gpointer user_data);
 extern void gui_batch_filetype_combobox_changed(gpointer active, gpointer user_data);
-extern void gui_export_changed_helper(GtkLabel *label);
-extern void gui_export_directory_entry_changed(GtkEntry *entry, gpointer user_data);
-extern void gui_export_filename_entry_changed(GtkComboBox *combobox, gpointer user_data);
-extern GtkWidget *gui_preferences_make_cms_page();
 extern gboolean window_key_press_event(GtkWidget *widget, GdkEventKey *event);
 extern void pos_menu_below_widget(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data);
 extern GtkWidget *gui_framed(GtkWidget *widget, const gchar *title, GtkShadowType shadowtype);
 extern GtkWidget *gui_aligned(GtkWidget *widget, const gfloat xalign, const gfloat yalign, const gfloat xscale, const gfloat yscale);
 extern GdkPixbuf *cairo_convert_to_pixbuf (cairo_surface_t *surface);
+extern void gui_set_block_keyboard(gboolean block_keyboard);
 
 /**
  * Build and show a popup-menu
@@ -60,9 +67,6 @@ typedef enum {
 
 extern void gui_select_theme(RS_THEME theme);
 
-extern GtkWidget *gui_dialog_make_from_text(const gchar *stock_id, gchar *primary_text, gchar *secondary_text);
-extern GtkWidget *gui_dialog_make_from_widget(const gchar *stock_id, gchar *primary_text, GtkWidget *widget);
-
 /**
  * Creates a new GtkButton widget.
  * @param stock_id A stock id registered with GTK+
@@ -79,3 +83,9 @@ extern GtkWidget *gui_button_new_from_stock_with_label(const gchar *stock_id, co
  * @return A new GtkLabel
  */
 extern GtkWidget *gui_label_new_with_mouseover(const gchar *normal_text, const gchar *hover_text);
+
+extern void gui_box_toggle_callback(GtkExpander *expander, gchar *key);
+#ifndef WIN32
+extern void gui_box_notify(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data);
+#endif
+extern GtkWidget * gui_box(const gchar *title, GtkWidget *in, gchar *key, gboolean default_expanded);
