@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
- * Anders Kvist <akv@lnxbx.dk>
+ * * Copyright (C) 2006-2011 Anders Brander <anders@brander.dk>, 
+ * * Anders Kvist <akv@lnxbx.dk> and Klaus Post <klauspost@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,4 +46,23 @@ extern void rs_core_action_group_activate(const gchar *name);
  */
 extern void rs_core_action_group_set_visibility(const gchar *name, gboolean visibility);
 
+/**
+ * Add actions to global action group, see documentation for gtk_action_group_add_actions
+ */
+extern void rs_core_action_group_add_actions(const GtkActionEntry *entries, guint n_entries, gpointer user_data);
+
+/**
+ * Add radio action to global action group, see documentation for gtk_action_group_add_radio_actions()
+ */
+extern void rs_core_action_group_add_radio_actions(const GtkRadioActionEntry *entries, guint n_entries, gint value, GCallback on_change, gpointer user_data);
+
+/**
+ * Get a GtkAction by name
+ * @param name The name of the action
+ * @return A GtkAction or NULL (should not be unreffed)
+ */
+extern GtkAction *rs_core_action_group_get_action(const gchar *name);
+
+extern void rs_core_actions_update_menu_items(RS_BLOB *rs);
+
 #endif /* RS_ACTIONS_H */
