@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
- * Anders Kvist <akv@lnxbx.dk>
+ * * Copyright (C) 2006-2011 Anders Brander <anders@brander.dk>,
+ * * Anders Kvist <akv@lnxbx.dk> and Klaus Post <klauspost@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,25 +17,24 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include <rawstudio.h>
 #include <glib.h>
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <config.h>
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
-#include "rawstudio.h"
+#include "application.h"
 #include "rs-batch.h"
 #include "conf_interface.h"
 #include "gettext.h"
 #include "gtk-helper.h"
+#include "gtk-interface.h"
 #include "filename.h"
 #include "rs-cache.h"
-#include "rs-color-transform.h"
-#include "rs-image.h"
 #include "rs-photo.h"
-#include "rs-metadata.h"
-#include "rs-utils.h"
 #include "rs-actions.h"
+#include "rs-store.h"
 
 extern GtkWindow *rawstudio_window;
 
@@ -143,11 +142,12 @@ batch_queue_load(RS_QUEUE *queue)
 	return;
 }
 
-RS_QUEUE* rs_batch_new_queue(void)
+RS_QUEUE* rs_batch_new_queue(RS_BLOB *rs)
 {
 	gchar *tmp;
 	RS_QUEUE *queue = g_new(RS_QUEUE, 1);
-	RS_FILETYPE *filetype;
+	queue->rs = rs;
+	queue->output = NULL;
 
 	queue->list = GTK_TREE_MODEL(gtk_list_store_new(5, G_TYPE_STRING,G_TYPE_STRING,
 									G_TYPE_INT,G_TYPE_STRING, GDK_TYPE_PIXBUF));
@@ -166,8 +166,6 @@ RS_QUEUE* rs_batch_new_queue(void)
 		queue->filename = rs_conf_get_string(CONF_BATCH_FILENAME);
 	}
 
-	rs_conf_get_filetype(CONF_BATCH_FILETYPE, &filetype);
-	queue->filetype = filetype->filetype;
 	queue->size_lock = LOCK_SCALE;
 	queue->size = 100;
 	queue->size_window = NULL;
@@ -320,6 +318,7 @@ rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_
 
 	batch_queue_save(queue);
 
+	rs_core_actions_update_menu_items(queue->rs); /* FIXME: should be done with a signal */
 	batch_queue_update_sensivity(queue);
 
 	return ret;
@@ -333,7 +332,7 @@ rs_batch_exists_in_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id
 
 	gchar *filename_temp;
 	gint setting_id_temp;
-	
+
 	gtk_tree_model_get_iter_first(queue->list, &iter);
 
 	if (gtk_list_store_iter_is_valid(GTK_LIST_STORE(queue->list), &iter))
@@ -376,7 +375,6 @@ void
 rs_batch_process(RS_QUEUE *queue)
 {
 	RS_PHOTO *photo = NULL;
-	RS_IMAGE16 *image;
 	GtkTreeIter iter;
 	gchar *filename_in;
 	gint setting_id;
@@ -384,7 +382,7 @@ rs_batch_process(RS_QUEUE *queue)
 	GdkPixbuf *pixbuf = NULL;
 	gint width = -1, height = -1;
 	gdouble scale = -1.0;
-	gchar *parsed_filename, *basename;
+	gchar *parsed_filename, *basename, *parsed_dir;
 	GString *filename;
 	GString *status = g_string_new(NULL);
 	GtkWidget *window;
@@ -392,37 +390,70 @@ rs_batch_process(RS_QUEUE *queue)
 	GtkWidget *vbox = gtk_vbox_new(FALSE, 4);
 	GtkWidget *cancel;
 	gboolean abort_render = FALSE;
-	gboolean fullscreen = FALSE;
-	RSColorTransform *rct = rs_color_transform_new();
 	GTimeVal start_time;
 	GTimeVal now_time = {0,0};
 	gint time, eta;
 	GtkWidget *eta_label = gtk_label_new(NULL);
-	gchar *eta_text;
+	gchar *eta_text, *title_text;
 	gint h = 0, m = 0, s = 0;
 	gint done = 0, left = 0;
+	RSFilter *finput = rs_filter_new("RSInputImage16", NULL);
+	RSFilter *fdemosaic = rs_filter_new("RSDemosaic", finput);
+	RSFilter *ffujirotate = rs_filter_new("RSFujiRotate", fdemosaic);
+	RSFilter *flensfun = rs_filter_new("RSLensfun", ffujirotate);
+	RSFilter *frotate = rs_filter_new("RSRotate", flensfun);
+	RSFilter *fcrop = rs_filter_new("RSCrop", frotate);
+	RSFilter *fcache = rs_filter_new("RSCache", fcrop);
+	RSFilter *fresample= rs_filter_new("RSResample", fcache);
+	RSFilter *ftransform_input = rs_filter_new("RSColorspaceTransform", fresample);
+	RSFilter *fdcp= rs_filter_new("RSDcp", ftransform_input);
+	RSFilter *fdenoise= rs_filter_new("RSDenoise", fdcp);
+	RSFilter *ftransform_display = rs_filter_new("RSColorspaceTransform", fdenoise);
+	RSFilter *fend = ftransform_display;
+	RSFilterResponse *filter_response;
+	RSColorSpace *display_color_space;
+	gchar* cs_name;
 
-	/* Initialize dimensions */
-	switch (queue->size_lock)
+	if ((cs_name = rs_conf_get_string("display-colorspace")))
+		display_color_space = rs_color_space_new_singleton(cs_name);
+	else
+		display_color_space = rs_color_space_new_singleton("RSSrgb");
+
+	/* FIXME: This is just a temporary hack to make batch work */
+#if 0
 	{
-		case LOCK_SCALE:
-			scale = queue->scale/100.0;
-			break;
-		case LOCK_WIDTH:
-			width = queue->width;
-			break;
-		case LOCK_HEIGHT:
-			height = queue->height;
-			break;
-		case LOCK_BOUNDING_BOX:
-			width = queue->width;
-			height = queue->height;
-			break;
-	}
+		RSIccProfile *profile;
 
+		profile = NULL;
+		gchar *profile_filename = rs_conf_get_cms_profile(CMS_PROFILE_INPUT);
+		if (profile_filename)
+		{
+			profile = rs_icc_profile_new_from_file(profile_filename);
+			g_free(profile_filename);
+		}
+		/*if (!profile)
+	        profile = rs_icc_profile_new_from_file(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "profiles" G_DIR_SEPARATOR_S "generic_camera_profile.icc");*/
+	    g_object_set(finput, "icc-profile", profile, NULL);
+	    g_object_unref(profile);
+
+		profile = NULL;
+		profile_filename = rs_conf_get_cms_profile(CMS_PROFILE_EXPORT);
+		if (profile_filename)
+		{
+			profile = rs_icc_profile_new_from_file(profile_filename);
+			g_free(profile_filename);
+		}
+		if (!profile)
+			profile = rs_icc_profile_new_from_file(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "profiles" G_DIR_SEPARATOR_S "sRGB.icc");
+		g_object_set(fend, "icc-profile", profile, NULL);
+	    g_object_unref(profile);
+	}
+#endif
+	gdk_threads_enter();
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_transient_for(GTK_WINDOW(window), rawstudio_window);
-	gtk_window_set_title(GTK_WINDOW(window), _("Processing photos"));
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
+	gtk_window_set_destroy_with_parent(GTK_WINDOW(window), TRUE);
 	gtk_window_resize(GTK_WINDOW(window), 250, 250);
 	g_signal_connect((gpointer) window, "delete_event", G_CALLBACK(window_destroy), &abort_render);
 
@@ -435,26 +466,19 @@ rs_batch_process(RS_QUEUE *queue)
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), eta_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), cancel, FALSE, FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 15);
 
-	/* Remember fullscreen state */
-	if (gdk_window_get_state((GTK_WIDGET(rawstudio_window))->window) & GDK_WINDOW_STATE_FULLSCREEN)
-	{
-		fullscreen = TRUE;
-		gtk_window_unfullscreen(rawstudio_window);
-	}
-	gtk_widget_hide(GTK_WIDGET(rawstudio_window));
 	gtk_widget_show_all(window);
 	while (gtk_events_pending()) gtk_main_iteration();
 
 	g_mkdir_with_parents(queue->directory, 00755);
 
 	g_get_current_time(&start_time);
-	left = rs_batch_num_entries(queue);
 
 	while(gtk_tree_model_get_iter_first(queue->list, &iter) && (!abort_render))
 	{
-		done++;
-		if (now_time.tv_sec > 0)
+		left = rs_batch_num_entries(queue);
+		if (done > 0 && now_time.tv_sec > 0)
 		{
 			time = (gint) (now_time.tv_sec-start_time.tv_sec);
 			eta = (time/done)*left;
@@ -465,15 +489,19 @@ rs_batch_process(RS_QUEUE *queue)
 			s = eta;
 
 			eta_text = g_strdup_printf(_("Time left: %dh %dm %ds"), h, m, s);
+			title_text = g_strdup_printf(_("Processing Image %d/%d"), done+1, done+left);
 		}
 		else
 		{
 			eta_text = g_strdup(_("Time left: ..."));
+			title_text = g_strdup_printf(_("Processing Image 1/%d."), left);
 		}
-		left--;
 
+		gtk_window_set_title(GTK_WINDOW(window), title_text);
 		gtk_label_set_text(GTK_LABEL(eta_label), eta_text);
 		g_free(eta_text);
+		g_free(title_text);
+		done++;
 
 		gtk_tree_model_get(queue->list, &iter,
 			RS_QUEUE_ELEMENT_FILENAME, &filename_in,
@@ -485,58 +513,130 @@ rs_batch_process(RS_QUEUE *queue)
 		while (gtk_events_pending()) gtk_main_iteration();
 		g_free(basename);
 
-		photo = rs_photo_load_from_file(filename_in, FALSE);
+		gdk_threads_leave();
+		photo = rs_photo_load_from_file(filename_in);
 		if (photo)
 		{
-			rs_image16_demosaic(photo->input, RS_DEMOSAIC_PPG);
 			rs_metadata_load_from_file(photo->metadata, filename_in);
-			filename = g_string_new(queue->directory);
-			g_string_append(filename, G_DIR_SEPARATOR_S);
-			g_string_append(filename, queue->filename);
-			g_string_append(filename, ".");
-
-			switch (queue->filetype)
-			{
-				case FILETYPE_JPEG:
-					g_string_append(filename, "jpg");
-					break;
-				case FILETYPE_PNG:
-					g_string_append(filename, "png");
-					break;
-				case FILETYPE_TIFF8:
-					g_string_append(filename, "tif");
-					break;
-				case FILETYPE_TIFF16:
-					g_string_append(filename, "tif");
-					break;
-			}
-
 			rs_cache_load(photo);
 
+			/* Build new filename */
+			if (NULL == g_strrstr(queue->filename, "%p"))
+			{
+				filename = g_string_new(queue->directory);
+				g_string_append(filename, G_DIR_SEPARATOR_S);
+				g_string_append(filename, queue->filename);
+			} 
+			else
+				filename = g_string_new(queue->filename);
+			
+			g_string_append(filename, ".");
+			g_string_append(filename, rs_output_get_extension(queue->output));
 			parsed_filename = filename_parse(filename->str, filename_in, setting_id);
+			
+			/* Create directory, if it doesn't exist */
+			parsed_dir = g_path_get_dirname(parsed_filename);
+			if (FALSE == g_file_test(parsed_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+				if (g_mkdir_with_parents(parsed_dir, 0x1ff))
+				{
+					gdk_threads_enter();
+					gui_status_notify(_("Could not create output directory."));
+					break;
+				}
 
-			image = rs_image16_transform(photo->input, NULL,
-				NULL, NULL, photo->crop, 200, 200, TRUE, -1.0,
-				photo->angle, photo->orientation, NULL);
-			if (pixbuf) g_object_unref(pixbuf);
-			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, image->w, image->h);
+			GList *filters = g_list_append(NULL, fend);
+			rs_photo_apply_to_filters(photo, filters, setting_id);
+			g_list_free(filters);
+
+			rs_filter_set_recursive(fend,
+				"image", photo->input_response,
+				"filename", photo->filename,
+				"bounding-box", TRUE,
+				"width", 250,
+				"height", 250,
+				NULL);
 
 			/* Render preview image */
-			rs_color_transform_set_from_settings(rct, photo->settings[setting_id], MASK_ALL);
-			rs_color_transform_transform(rct, image->w, image->h, image->pixels,
-				image->rowstride, gdk_pixbuf_get_pixels(pixbuf),
-				gdk_pixbuf_get_rowstride(pixbuf));
-			gtk_image_set_from_pixbuf((GtkImage *) preview, pixbuf);
-			rs_image16_free(image);
+			RSFilterRequest *request = rs_filter_request_new();
+			rs_filter_request_set_quick(RS_FILTER_REQUEST(request), FALSE);
+			/* FIXME: Should be set to output colorspace, not forced to sRGB */
 
+			rs_filter_param_set_object(RS_FILTER_PARAM(request), "colorspace", display_color_space);	
+			filter_response = rs_filter_get_image8(fend, request);
+			pixbuf = rs_filter_response_get_image8(filter_response);
+			gdk_threads_enter();
+			if (pixbuf)
+			{
+				gtk_image_set_from_pixbuf(GTK_IMAGE(preview), pixbuf);
+				g_object_unref(pixbuf);
+			}
+			g_object_unref(request);
+			g_object_unref(filter_response);
+
+			if (left > 1)
+			{
+				GtkTreeIter iter2 = iter;
+				if (gtk_tree_model_iter_next (queue->list, &iter2))
+				{
+					gtk_tree_model_get(queue->list, &iter2, RS_QUEUE_ELEMENT_FILENAME, &filename_in, -1);
+					rs_io_idle_prefetch_file(filename_in, 0xC01A);
+				}
+			}
+			/* Build text for small preview-window */
 			basename = g_path_get_basename(parsed_filename);
 			g_string_printf(status, _("Saving %s ..."), basename);
 			gtk_label_set_text(GTK_LABEL(label), status->str);
-			while (gtk_events_pending()) gtk_main_iteration();
+			while (gtk_events_pending())
+				gtk_main_iteration();
 			g_free(basename);
+			gdk_threads_leave();
+
+			width = 65535;
+			height = 65535;
+			/* Calculate new size */
+			switch (queue->size_lock)
+			{
+				case LOCK_SCALE:
+					scale = queue->scale/100.0;
+					rs_filter_get_size_simple(fcrop, RS_FILTER_REQUEST_QUICK, &width, &height);
+					width = (gint) (((gdouble) width) * scale);
+					height = (gint) (((gdouble) height) * scale);
+					break;
+				case LOCK_WIDTH:
+					width = queue->width;
+					break;
+				case LOCK_HEIGHT:
+					height = queue->height;
+					break;
+				case LOCK_BOUNDING_BOX:
+					width = queue->width;
+					height = queue->height;
+					break;
+			}
+			rs_filter_set_recursive(fend,
+				"width", width,
+				"height", height,
+				NULL);
+
+			/* Save the image */
+			if (g_object_class_find_property(G_OBJECT_GET_CLASS(queue->output), "filename"))
+				g_object_set(queue->output, "filename", parsed_filename, NULL);
+
+			rs_output_set_from_conf(queue->output, "batch");
+
+			g_assert(RS_IS_OUTPUT(queue->output));
+			g_assert(RS_IS_FILTER(fend));
+
+			gboolean exported = rs_output_execute(queue->output, fend);
+			gdk_threads_enter();
+			if (exported)
+				rs_store_set_flags(NULL, photo->filename, NULL, NULL, &exported);
+			else
+			{
+				gui_status_notify(_("Could not export photo."));
+				break;
+			}
 
-			rs_photo_save(photo, parsed_filename, queue->filetype,
-				width, height, TRUE, scale, setting_id, queue->cms);
 			g_free(parsed_filename);
 			g_string_free(filename, TRUE);
 			g_object_unref(photo);
@@ -547,15 +647,23 @@ rs_batch_process(RS_QUEUE *queue)
 
 		g_get_current_time(&now_time);
 	}
-	g_object_unref(rct);
 	gtk_widget_destroy(window);
 
-	/* Restore fullscreen state if needed */
-	if (fullscreen)	
-		gtk_window_fullscreen(rawstudio_window);
-	gtk_widget_show_all(GTK_WIDGET(rawstudio_window));
-
 	batch_queue_update_sensivity(queue);
+	gdk_threads_leave();
+
+	g_object_unref(finput);
+	g_object_unref(fdemosaic);
+	g_object_unref(ffujirotate);
+	g_object_unref(flensfun);
+	g_object_unref(frotate);
+	g_object_unref(fcrop);
+	g_object_unref(fcache);
+	g_object_unref(fresample);
+	g_object_unref(fdcp);
+	g_object_unref(fdenoise);
+	g_object_unref(ftransform_input);
+	g_object_unref(ftransform_display);
 }
 
 static void
@@ -571,7 +679,7 @@ make_batchview(RS_QUEUE *queue)
 	GtkWidget *view;
 	GtkCellRenderer *renderer_text, *renderer_pixbuf;
 	GtkTreeViewColumn *column_filename, *column_setting_id, *column_pixbuf;
-	
+
 	scroller = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
 		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
@@ -579,6 +687,7 @@ make_batchview(RS_QUEUE *queue)
 	view = gtk_tree_view_new_with_model(queue->list);
 	queue->view = GTK_TREE_VIEW(view);
 
+	gtk_tree_view_set_reorderable(queue->view, TRUE);
 	gtk_container_add (GTK_CONTAINER (scroller), view);
 
 	renderer_text = gtk_cell_renderer_text_new();
@@ -633,6 +742,7 @@ batch_button_remove_clicked(GtkWidget *button, RS_QUEUE *queue)
 			batch_queue_save(queue);
 		}
 	}
+	rs_core_actions_update_menu_items(queue->rs); /* FIXME: should be done with a signal */
 	batch_queue_update_sensivity(queue);
 	return;
 }
@@ -661,11 +771,13 @@ batch_queue_update_sensivity(RS_QUEUE *queue)
 	/* If we have any entries, enable "Start" and "Remove all" */
 	if (gtk_tree_model_get_iter_first(queue->list, &iter))
 	{
+		rs_core_action_group_set_sensivity("ProcessBatch", TRUE);
 		gtk_widget_set_sensitive(queue->start_button, TRUE);
 		gtk_widget_set_sensitive(queue->remove_all_button, TRUE);
 	}
 	else
 	{
+		rs_core_action_group_set_sensivity("ProcessBatch", FALSE);
 		gtk_widget_set_sensitive(queue->start_button, FALSE);
 		gtk_widget_set_sensitive(queue->remove_all_button, FALSE);
 	}
@@ -718,8 +830,13 @@ static void
 filetype_changed(gpointer active, gpointer user_data)
 {
 	RS_QUEUE *queue = (RS_QUEUE *) user_data;
-	RS_FILETYPE *filetype = (RS_FILETYPE *) active;
-	queue->filetype = filetype->filetype;
+	GType filetype = GPOINTER_TO_INT(active);
+
+	if (!filetype)
+		return;
+	if (queue->output)
+		g_object_unref(queue->output);
+	queue->output = rs_output_new(g_type_name(filetype));
 }
 
 static void
@@ -795,6 +912,25 @@ size_close_clicked(GtkButton *button, RS_QUEUE *queue)
 	gtk_widget_hide(queue->size_window);
 }
 
+static void
+edit_settings_clicked(GtkButton *button, RS_QUEUE *queue)
+{
+	RSOutput *output = queue->output;
+	GtkWidget *dialog = gtk_dialog_new_with_buttons (_("Edit output settings"),
+							 NULL,
+							 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+							 GTK_STOCK_OK,
+							 GTK_RESPONSE_ACCEPT,
+							 NULL);
+	g_signal_connect_swapped (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
+
+	GtkWidget *content = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+	GtkWidget *settings = rs_output_get_parameter_widget(output, "batch");
+	gtk_container_add (GTK_CONTAINER (content), settings);
+	gtk_widget_show_all(dialog);
+	gtk_dialog_run(GTK_DIALOG(dialog));
+}
+
 static void
 batch_size_selection(GtkWidget *button, RS_QUEUE *queue)
 {
@@ -832,6 +968,7 @@ batch_size_selection(GtkWidget *button, RS_QUEUE *queue)
 	gtk_widget_show(gui_confbox_get_widget(lockbox));
 
 	gtk_box_pack_start (GTK_BOX (vbox), gui_confbox_get_widget(lockbox), FALSE, TRUE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
 
 	/* Spinners */
 	table = gtk_table_new(3, 3, FALSE);
@@ -926,6 +1063,7 @@ make_batch_options(RS_QUEUE *queue)
 	GtkWidget *filename;
 	RS_CONFBOX *filetype_confbox;
 	GtkWidget *size_button;
+	gpointer active;
 
 	chooser = gtk_file_chooser_button_new(_("Choose output directory"),
 		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
@@ -944,6 +1082,15 @@ make_batch_options(RS_QUEUE *queue)
 	gui_confbox_set_callback(filetype_confbox, queue, filetype_changed);
 	gtk_box_pack_start (GTK_BOX (vbox), gui_confbox_get_widget(filetype_confbox), FALSE, TRUE, 0);
 
+	active = gui_confbox_get_active(filetype_confbox);
+	if (!active)
+		active = GUINT_TO_POINTER(g_type_from_name("RSJpegfile"));
+	filetype_changed(active, queue);
+
+	GtkWidget *edit_settings = gtk_button_new_with_label(_("Edit output settings"));
+	g_signal_connect ((gpointer) edit_settings, "clicked", G_CALLBACK (edit_settings_clicked), queue);
+	gtk_box_pack_start (GTK_BOX (vbox), edit_settings, FALSE, TRUE, 0);
+
 	/* Export size */
 	hbox = gtk_hbox_new(FALSE, 1);
 	queue->size_label = gtk_label_new(NULL);
