@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
- * Anders Kvist <akv@lnxbx.dk>
+ * * Copyright (C) 2006-2011 Anders Brander <anders@brander.dk>, 
+ * * Anders Kvist <akv@lnxbx.dk> and Klaus Post <klauspost@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,18 +17,16 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include <rawstudio.h>
 #include <glib.h>
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
-#include "rawstudio.h"
+#include "application.h"
 #include "rs-cache.h"
 #include "rs-photo.h"
-#include "rs-utils.h"
 
 /* This will be written to XML files for making backward compatibility easier to implement */
-#define CACHEVERSION 2
-
-static guint rs_cache_load_setting(RSSettings *rss, xmlDocPtr doc, xmlNodePtr cur);
+#define CACHEVERSION 5
 
 gchar *
 rs_cache_get_name(const gchar *src)
@@ -55,7 +53,7 @@ rs_cache_get_name(const gchar *src)
 void
 rs_cache_save(RS_PHOTO *photo, const RSSettingsMask mask)
 {
-	gint id, i;
+	gint id;
 	xmlTextWriterPtr writer;
 	gchar *cachename;
 
@@ -76,6 +74,29 @@ rs_cache_save(RS_PHOTO *photo, const RSSettingsMask mask)
 		photo->orientation);
 	xmlTextWriterWriteFormatElement(writer, BAD_CAST "angle", "%f",
 		photo->angle);
+
+	RSDcpFile *dcp = rs_photo_get_dcp_profile(photo);
+	if (RS_IS_DCP_FILE(dcp))
+	{
+		const gchar *dcp_id = rs_dcp_get_id(RS_DCP_FILE(dcp));
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "dcp-profile", "%s",
+			dcp_id);
+	}
+
+	RSIccProfile *icc = rs_photo_get_icc_profile(photo);
+	if (RS_IS_ICC_PROFILE(icc))
+	{
+		const gchar *icc_filename;
+		g_object_get(icc, "filename", &icc_filename, NULL);
+		if (icc_filename)
+		{
+			gchar *basename = g_path_get_basename(icc_filename);
+			xmlTextWriterWriteFormatElement(writer, BAD_CAST "icc-profile", "%s",
+			basename);
+			g_free(basename);
+		}
+	}
+
 	if (photo->crop)
 	{
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "crop", "%d %d %d %d",
@@ -86,37 +107,7 @@ rs_cache_save(RS_PHOTO *photo, const RSSettingsMask mask)
 	{
 		xmlTextWriterStartElement(writer, BAD_CAST "settings");
 		xmlTextWriterWriteFormatAttribute(writer, BAD_CAST "id", "%d", id);
-		if (mask & MASK_EXPOSURE)
-			xmlTextWriterWriteFormatElement(writer, BAD_CAST "exposure", "%f",
-				photo->settings[id]->exposure);
-		if (mask & MASK_SATURATION)
-		xmlTextWriterWriteFormatElement(writer, BAD_CAST "saturation", "%f",
-			photo->settings[id]->saturation);
-		if (mask & MASK_HUE)
-		xmlTextWriterWriteFormatElement(writer, BAD_CAST "hue", "%f",
-			photo->settings[id]->hue);
-		if (mask & MASK_CONTRAST)
-		xmlTextWriterWriteFormatElement(writer, BAD_CAST "contrast", "%f",
-			photo->settings[id]->contrast);
-		if (mask & MASK_WARMTH)
-		xmlTextWriterWriteFormatElement(writer, BAD_CAST "warmth", "%f",
-			photo->settings[id]->warmth);
-		if (mask & MASK_TINT)
-		xmlTextWriterWriteFormatElement(writer, BAD_CAST "tint", "%f",
-			photo->settings[id]->tint);
-		if (mask & MASK_SHARPEN)
-		xmlTextWriterWriteFormatElement(writer, BAD_CAST "sharpen", "%f",
-			photo->settings[id]->sharpen);
-		if (mask & MASK_CURVE && photo->settings[id]->curve_nknots > 0)
-		{
-			xmlTextWriterStartElement(writer, BAD_CAST "curve");
-			xmlTextWriterWriteFormatAttribute(writer, BAD_CAST "num", "%d", photo->settings[id]->curve_nknots);
-			for(i=0;i<photo->settings[id]->curve_nknots;i++)
-				xmlTextWriterWriteFormatElement(writer, BAD_CAST "knot", "%f %f",
-					photo->settings[id]->curve_knots[i*2+0],
-					photo->settings[id]->curve_knots[i*2+1]);
-			xmlTextWriterEndElement(writer);
-		}
+		rs_cache_save_settings(photo->settings[id], mask, writer);
 		xmlTextWriterEndElement(writer);
 	}
 	xmlTextWriterEndDocument(writer);
@@ -125,8 +116,56 @@ rs_cache_save(RS_PHOTO *photo, const RSSettingsMask mask)
 	return;
 }
 
-static guint
-rs_cache_load_setting(RSSettings *rss, xmlDocPtr doc, xmlNodePtr cur)
+void
+rs_cache_save_settings(RSSettings *rss, const RSSettingsMask mask, xmlTextWriterPtr writer)
+{
+	if (mask & MASK_EXPOSURE)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "exposure", "%f", rss->exposure);
+	if (mask & MASK_SATURATION)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "saturation", "%f", rss->saturation);
+	if (mask & MASK_HUE)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "hue", "%f", rss->hue);
+	if (mask & MASK_CONTRAST)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "contrast", "%f", rss->contrast);
+	if (mask & MASK_WARMTH)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "warmth", "%f", rss->dcp_temp);
+	if (mask & MASK_TINT)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "tint", "%f", rss->dcp_tint);
+	if (mask & MASK_WB && rss->wb_ascii)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "wb_ascii", "%s", rss->wb_ascii);
+	if (mask & MASK_SHARPEN)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "sharpen", "%f", rss->sharpen);
+	if (mask & MASK_DENOISE_LUMA)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "denoise_luma", "%f", rss->denoise_luma);
+	if (mask & MASK_DENOISE_CHROMA)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "denoise_chroma", "%f", rss->denoise_chroma);
+	if (mask & MASK_CHANNELMIXER)
+	{
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "channelmixer_red", "%f", rss->channelmixer_red);
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "channelmixer_green", "%f", rss->channelmixer_green);
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "channelmixer_blue", "%f", rss->channelmixer_blue);
+	}
+	if (mask & MASK_TCA_KR)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "tca_kr", "%f", rss->tca_kr);
+	if (mask & MASK_TCA_KB)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "tca_kb", "%f", rss->tca_kb);
+	if (mask & MASK_VIGNETTING)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "vignetting", "%f", rss->vignetting);
+	if (mask & MASK_CURVE && rss->curve_nknots > 0)
+	{
+		gint i;
+		xmlTextWriterStartElement(writer, BAD_CAST "curve");
+		xmlTextWriterWriteFormatAttribute(writer, BAD_CAST "num", "%d", rss->curve_nknots);
+		for(i=0;i<rss->curve_nknots;i++)
+			xmlTextWriterWriteFormatElement(writer, BAD_CAST "knot", "%f %f",
+				rss->curve_knots[i*2+0],
+				rss->curve_knots[i*2+1]);
+		xmlTextWriterEndElement(writer);
+	}
+}
+
+guint
+rs_cache_load_setting(RSSettings *rss, xmlDocPtr doc, xmlNodePtr cur, gint version)
 {
 	RSSettingsMask mask = 0;
 	xmlChar *val;
@@ -157,19 +196,105 @@ rs_cache_load_setting(RSSettings *rss, xmlDocPtr doc, xmlNodePtr cur)
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "warmth")))
 		{
-			mask |= MASK_WARMTH;
-			target = &rss->warmth;
+			if ( version <= 4)
+			{
+				mask |= MASK_WARMTH;
+				target = &rss->warmth;
+				rss->recalc_temp = TRUE;
+			}
+			else
+			{
+				mask |= MASK_WARMTH;
+				target = &rss->dcp_temp;
+			}
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "tint")))
 		{
-			mask |= MASK_TINT;
-			target = &rss->tint;
+			if ( version <= 4)
+			{
+				mask |= MASK_TINT;
+				target = &rss->tint;
+				rss->recalc_temp = TRUE;
+			}
+			else
+			{
+				mask |= MASK_TINT;
+				target = &rss->dcp_tint;
+			}
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "wb_ascii")))
+		{
+			mask |= MASK_WB;
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			rss->wb_ascii = g_strdup((gchar *) val);
+			xmlFree(val);
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "sharpen")))
 		{
 			mask |= MASK_SHARPEN;
 			target = &rss->sharpen;
 		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "denoise_luma")))
+		{
+			mask |= MASK_DENOISE_LUMA;
+			target = &rss->denoise_luma;
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "denoise_chroma")))
+		{
+			mask |= MASK_DENOISE_CHROMA;
+			target = &rss->denoise_chroma;
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "channelmixer_red")))
+		{
+			mask |= MASK_CHANNELMIXER_RED;
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			rss->channelmixer_red =  rs_atof((gchar *) val);
+			xmlFree(val);
+
+			if (version < 4)
+				rss->channelmixer_red *= 3.0;
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "channelmixer_green")))
+		{
+			mask |= MASK_CHANNELMIXER_GREEN;
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			rss->channelmixer_green =  rs_atof((gchar *) val);
+			xmlFree(val);
+
+			if (version < 4)
+				rss->channelmixer_green *= 3.0;
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "channelmixer_blue")))
+		{
+			mask |= MASK_CHANNELMIXER_BLUE;
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			rss->channelmixer_blue =  rs_atof((gchar *) val);
+			xmlFree(val);
+
+			if (version < 4)
+				rss->channelmixer_blue *= 3.0;
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "tca_kr")))
+		{
+			mask |= MASK_TCA_KR;
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			rss->tca_kr =  rs_atof((gchar *) val);
+			xmlFree(val);
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "tca_kb")))
+		{
+			mask |= MASK_TCA_KB;
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			rss->tca_kb =  rs_atof((gchar *) val);
+			xmlFree(val);
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "vignetting")))
+		{
+			mask |= MASK_VIGNETTING;
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			rss->vignetting =  rs_atof((gchar *) val);
+			xmlFree(val);
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "curve")))
 		{
 			gchar **vals;
@@ -257,48 +382,85 @@ rs_cache_load(RS_PHOTO *photo)
 		if ((!xmlStrcmp(cur->name, BAD_CAST "settings")))
 		{
 			val = xmlGetProp(cur, BAD_CAST "id");
-			id = atoi((gchar *) val);
+			id = (val) ? atoi((gchar *) val) : 0;
 			xmlFree(val);
 			if (id>2) id=0;
 			if (id<0) id=0;
 			settings = rs_settings_new();
-			mask |= rs_cache_load_setting(settings, doc, cur->xmlChildrenNode);
+			mask |= rs_cache_load_setting(settings, doc, cur->xmlChildrenNode, version);
 			rs_photo_apply_settings(photo, id, settings, MASK_ALL);
 			g_object_unref(settings);
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "priority")))
 		{
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			photo->priority = atoi((gchar *) val);
-			xmlFree(val);
+			if (val)
+			{
+				photo->priority = atoi((gchar *) val);
+				xmlFree(val);
+			}
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "orientation")))
 		{
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			photo->orientation = atoi((gchar *) val);
-			xmlFree(val);
+			if (val)
+			{
+				photo->orientation = atoi((gchar *) val);
+				xmlFree(val);
+			}
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "angle")))
 		{
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			photo->angle = rs_atof((gchar *) val);
-			xmlFree(val);
+			if (val)
+			{
+				photo->angle = rs_atof((gchar *) val);
+				xmlFree(val);
+			}
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "exported")))
 		{
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			if (g_ascii_strcasecmp((gchar *) val, "yes")==0)
-				photo->exported = TRUE;
-			xmlFree(val);
+			if (val)
+			{
+				if (g_ascii_strcasecmp((gchar *) val, "yes")==0)
+					photo->exported = TRUE;
+				xmlFree(val);
+			}
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "dcp-profile")))
+		{
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			if (val)
+			{
+				RSProfileFactory *factory = rs_profile_factory_new_default();
+				RSDcpFile *dcp = rs_profile_factory_find_from_id(factory, (gchar *) val);
+				if (dcp)
+					rs_photo_set_dcp_profile(photo, dcp);
+				xmlFree(val);
+			}
+		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "icc-profile")))
+		{
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			if (val)
+			{
+				RSProfileFactory *factory = rs_profile_factory_new_default();
+				RSIccProfile *icc = rs_profile_factory_find_icc_from_filename(factory, (gchar *) val);
+				if (icc)
+					rs_photo_set_icc_profile(photo, icc);
+				xmlFree(val);
+			}
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "crop")))
 		{
 			RS_RECT *crop = g_new0(RS_RECT, 1);
-			gchar **vals;
+			gchar **vals = NULL;
 
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			vals = g_strsplit((gchar *)val, " ", 4);
-			if (vals[0])
+			if (val)
+				vals = g_strsplit((gchar *)val, " ", 4);
+			if (val && vals[0])
 			{
 				crop->x1 = atoi((gchar *) vals[0]);
 				if (vals[1])
@@ -344,6 +506,9 @@ rs_cache_load_quick(const gchar *filename, gint *priority, gboolean *exported)
 	xmlChar *val;
 	gchar *cachename;
 
+	if (priority) *priority = PRIO_U;
+	if (exported) *exported = FALSE;
+
 	if (!filename)
 		return;
 
@@ -364,8 +529,6 @@ rs_cache_load_quick(const gchar *filename, gint *priority, gboolean *exported)
 	if(doc==NULL)
 		return;
 
-	if (exported) *exported = FALSE;
-
 	cur = xmlDocGetRootElement(doc);
 
 	cur = cur->xmlChildrenNode;
