@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2009 Anders Brander <anders@brander.dk> and 
- * Anders Kvist <akv@lnxbx.dk>
+ * Copyright (C) 2006-2011 Anders Brander <anders@brander.dk>,
+ * Anders Kvist <akv@lnxbx.dk> and Klaus Post <klauspost@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,6 @@
 #ifndef RS_EXTERNAL_EDITOR_H
 #define RS_EXTERNAL_EDITOR_H
 
-extern gboolean rs_external_editor_gimp(RS_PHOTO *photo, guint snapshot, void *cms);
+extern gboolean rs_external_editor_gimp(RS_PHOTO *photo, RSFilter *prior_to_resample, guint snapshot);
 
 #endif /* RS_EXTERNAL_EDITOR_H */
