@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
- * Anders Kvist <akv@lnxbx.dk>
+ * * Copyright (C) 2006-2011 Anders Brander <anders@brander.dk>, 
+ * * Anders Kvist <akv@lnxbx.dk> and Klaus Post <klauspost@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -54,6 +54,11 @@ rs_store_new(void);
 #define RS_IS_STORE_CLASS(obj)    (G_TYPE_CHECK_CLASS_TYPE ((obj), RS_STORE_TYPE_WIDGET))
 #define RS_STORE_GET_CLASS        (G_TYPE_INSTANCE_GET_CLASS ((obj), RS_STORE_TYPE_WIDGET, RSStoreClass))
 
+/* Classes to user for io-system */ 
+#define PRELOAD_CLASS (82764283)
+#define METADATA_CLASS (542344)
+#define RESTORE_TAGS_CLASS (4845658)
+
 /**
  * Load thumbnails from a directory into the store
  * @param store A RSStore
@@ -76,13 +81,23 @@ extern gboolean
 rs_store_set_flags(RSStore *store, const gchar *filename, GtkTreeIter *iter,
 	const guint *priority, const gboolean *exported);
 
+/**
+ * Update thumbnail of a file in the store
+ * @param store A RSStore
+ * @param filename The name of the thumbnail to remove or NULL
+ * @param thumbnail A pixbuffer containing the updated thumbnail
+ */
+extern void
+rs_store_update_thumbnail(RSStore *store, const gchar *filename, GdkPixbuf *thumbnail);
+
 /**
  * Select one image
  * @param store A RSStore
  * @param name The filename to select
+ * @param deselect_others Should other images be de-selected.
  */
 extern gboolean
-rs_store_set_selected_name(RSStore *store, const gchar *filename);
+rs_store_set_selected_name(RSStore* store, const gchar* filename, gboolean deselect_others);
 
 /**
  * Remove thumbnail(s) from store
@@ -145,6 +160,16 @@ rs_store_get_iters_with_priority(RSStore *store, guint priority);
 extern gchar *
 rs_store_get_name(RSStore *store, GtkTreeIter *iter);
 
+/**
+ * Get the filename of the previous or next thumbnail
+ * @param store A RSStore
+ * @param current_filename Current filename or NULL if none
+ * @param direction 1: previous, 2: next
+ * @return filename of next or previous file, NULL if none.
+ */
+const gchar*
+rs_store_get_prevnext(RSStore *store, const gchar *current_filename, guint direction);
+
 /**
  * Selects the previous or next thumbnail
  * @param store A RSStore
@@ -220,4 +245,35 @@ rs_store_group_select_name(RSStore *store, const gchar *filename);
 extern void
 rs_store_group_ungroup_name(RSStore *store, const gchar *filename);
 
+/**
+ * Add a single image to the image store.
+ * @param store A RSStore
+ * @param fullname Full filename including path to the file.
+ */
+extern void
+rs_store_load_file(RSStore *store, gchar *fullname);
+
+extern void
+rs_store_set_iconview_size(RSStore *store, gint size);
+
+extern gint
+rs_store_get_iconview_size(RSStore *store);
+
+extern gboolean
+rs_store_set_open_selected(RSStore *store, gboolean open_selected);
+
+/**
+ * Return whether the file is assumed to be selected
+ * A special case is if NONE is selected, then we will return TRUE, since this will 
+ * actually assume that the current photo is selected.
+ * @param store A RSStore
+ * @param filename The filename of the photo
+ * @return TRUE if the photo is selected in the store, FALSE otherwise
+ */
+extern gboolean
+rs_store_is_photo_selected(RSStore *store, const gchar *filename);
+
+extern GtkIconView*
+rs_store_get_current_iconview(RSStore *store);
+
 #endif /* RS_STORE_H */
